package com.ydl.media.view

import android.annotation.SuppressLint
import android.content.Context
import android.text.TextUtils
import android.view.MotionEvent
import android.view.View
import android.view.ViewConfiguration
import android.view.WindowManager
import android.widget.FrameLayout
import com.ydl.media.R
import com.ydl.media.audio.AudioPlayer
import com.ydl.media.audio.OnPlayerEventListener
import com.ydl.media.audio.model.Music
import com.ydl.ydl_image.module.GlideApp
import com.ydl.ydl_image.transform.GlideRoundTransform
import com.ydl.ydlcommon.utils.statusBar.StatusBarUtil
import kotlinx.android.synthetic.main.item_playing_float_btn.view.*
import java.util.concurrent.CopyOnWriteArraySet
import kotlin.math.abs


class PlayerFloatView(private val ctx: Context) : FrameLayout(ctx) {
    private var mTouchDownX: Float = 0.toFloat()
    private var mTouchDownY: Float = 0.toFloat()
    private var mTouchX: Float = 0.toFloat()
    private var mTouchY: Float = 0.toFloat()

    private val mTouchSlop: Int = (ViewConfiguration.get(ctx).scaledTouchSlop + 8) * 3

    private val mWindowManager: WindowManager = ctx.getSystemService(Context.WINDOW_SERVICE) as WindowManager
    //全局变量，用以保存悬浮窗口的属性
    var mFloatViewParams = WindowManager.LayoutParams()

    private var mListeners: CopyOnWriteArraySet<FloatViewPlayListener> = CopyOnWriteArraySet()
    private var mStateChangeListener: OnPlayerEventListener? = null

    private val mStatusBarHeight: Int = StatusBarUtil.getStatusBarHeight(ctx)


    fun resetLayoutParams() {
        mFloatViewParams = WindowManager.LayoutParams()
    }

    init {
        View.inflate(context, R.layout.item_playing_float_btn, this)
        resetFloatView()
        if (!TextUtils.isEmpty(AudioPlayer.get().playMusic?.coverPath)) {
            GlideApp.with(context.applicationContext)
                .load(AudioPlayer.get().playMusic?.coverPath)
                .transform(GlideRoundTransform(context, 4))
                .error(R.drawable.ico_play_float_pic)
                .into(play_head)
        }
        cl.setOnClickListener {
            if (PlayerFloatHelper.isCanClick) {
                PlayerFloatHelper.startPlayingActivity(context)
            }
        }
        play_close.setOnClickListener {
            mListeners.forEach {
                it.onPauseClick()
            }
            PlayerFloatHelper.removeView(ctx)
            PlayerFloatHelper.playTempData.clear()
            AudioPlayer.get().stopPlayer()
        }
        play_state.setOnClickListener {
            if (AudioPlayer.get().isPlaying) {
                play_state.setImageResource(R.drawable.ico_yyfc_play)
                AudioPlayer.get().playPause()
                mListeners.forEach {
                    it.onPauseClick()
                }
            } else {
                play_state.setImageResource(R.drawable.ico_yyfc_pause)
                AudioPlayer.get().playPause()
                mListeners.forEach {
                    it.onStartClick()
                }
            }
        }
        if (mStateChangeListener == null) {
            mStateChangeListener = object : OnPlayerEventListener {

                override fun onComplete() {
                    updatePlayState()
                    mListeners.forEach {
                        it.onPlayFinish()
                    }
                }

                override fun onLoad(music: Music) {
                }

                override fun onPlayerStart() {
                }

                override fun onPlayerPause() {
                }

                @SuppressLint("SetTextI18n")
                override fun onPublish(percent: Int, currentPosition: Long) {
                    tv_now_playing_time.text = getStringTime(currentPosition.toInt())
                }

                override fun onBufferingUpdate(percent: Int) {
                }

                @SuppressLint("SetTextI18n")
                override fun onPrepared(duration: Long) {
                    updatePlayState()
                }
            }
            AudioPlayer.get().addOnPlayEventListener(mStateChangeListener!!)
        }
    }


    @SuppressLint("SetTextI18n")
    fun resetFloatView() {
        if (!TextUtils.isEmpty(AudioPlayer.get().playMusic?.coverPath)) {
            GlideApp.with(context.applicationContext)
                .load(AudioPlayer.get().playMusic?.coverPath)
                .transform(GlideRoundTransform(context, 4))
                .error(R.drawable.ico_play_float_pic)
                .into(play_head)
        }

        if (TextUtils.isEmpty(AudioPlayer.get().playMusic?.title)) {
            if (PlayerFloatHelper.playingType == PlayTypeEnum.PLAY_TYPE_FM) {
                tv_title.text = "心灵电台"
            }
            if (PlayerFloatHelper.playingType == PlayTypeEnum.PLAY_TYPE_CONFIDE) {
                tv_title.text = "壹点倾诉，心灵寄语"
            }
        } else {
            tv_title.text = AudioPlayer.get().playMusic?.title
        }
        tv_name.text = if (TextUtils.isEmpty(AudioPlayer.get().playMusic?.artist)) {
            "壹点灵"
        } else {
            AudioPlayer.get().playMusic?.artist
        }
        tv_duration.text = " / ${getStringTime(AudioPlayer.get().getDuration().toInt())}"
    }


    private fun getStringTime(time: Int): String {
        if (time <= 0) return "00:00"
        val min = time / 60000
        val ss = (time - min * 60000) / 1000

        var m = "" + min
        var s = "" + ss
        if (min < 10) {
            m = "0$m"
        }
        if (ss < 10) {
            s = "0$ss"
        }
        return "$m:$s"
    }

    override fun onInterceptTouchEvent(ev: MotionEvent): Boolean {
        when (ev.action) {
            MotionEvent.ACTION_DOWN -> {
                mTouchDownY = ev.y
            }
            MotionEvent.ACTION_MOVE -> {
                val moveY = ev.y
                if (abs(moveY - mTouchDownY) > mTouchSlop) {
                    return true
                }
            }
            MotionEvent.ACTION_UP -> {
                mTouchDownY = 0f
            }
        }
        return super.onInterceptTouchEvent(ev)
    }

    override fun onTouchEvent(event: MotionEvent): Boolean {
        //获取相对屏幕的坐标，即以屏幕左上角为原点
        mTouchX = event.rawX
        mTouchY = event.rawY
        when (event.action) {
            MotionEvent.ACTION_DOWN -> {
                //获取相对View的坐标，即以此View左上角为原点
                mTouchDownX = event.x
                mTouchDownY = event.y
            }
            MotionEvent.ACTION_MOVE -> updateFloatViewPosition()
            MotionEvent.ACTION_UP -> {
                updateFloatViewPosition()
                mTouchDownY = 0f
                mTouchDownX = mTouchDownY
            }
        }
        return super.onTouchEvent(event)
    }

    private fun updateFloatViewPosition() {
        //更新浮动窗口位置参数
        mFloatViewParams.x = (mTouchX - mTouchDownX).toInt()
        mFloatViewParams.y = (mTouchY - mTouchDownY - mStatusBarHeight).toInt()
        mWindowManager.updateViewLayout(this, mFloatViewParams)
    }

    fun updatePlayState() {
        if (AudioPlayer.get().isPlaying) {
            play_state.setImageResource(R.drawable.ico_yyfc_pause)
        } else {
            play_state.setImageResource(R.drawable.ico_yyfc_play)
            for (listener in mListeners) {
                listener.onPauseClick()
            }
        }
    }

    fun setPlayingState() {
        play_state.setImageResource(R.drawable.ico_yyfc_pause)
        resetFloatView()
    }

    fun onDestroy() {
        mListeners.clear()
        mStateChangeListener?.let { AudioPlayer.get().removeOnPlayEventListener(it) }
    }

    fun addFloatClickListener(floatClickListener: FloatViewPlayListener) {
        mListeners.add(floatClickListener)
    }

    fun removeFloatClickListener(floatClickListener: FloatViewPlayListener) {
        mListeners.remove(floatClickListener)
    }

    interface FloatViewPlayListener {

        fun onPauseClick()

        fun onStartClick()

        fun onPlayFinish()
    }

}